
FilterEngine = class();

function FilterEngine:Constructor(filterData)
	self.Data = filterData;
	self.Policies = {};

	-- keep blacklist policy on top
	table.insert(self.Policies, Policies.BlacklistPolicy(filterData));
	table.insert(self.Policies, Policies.DuplicationPolicy(filterData));
	table.insert(self.Policies, Policies.UserPolicy(filterData));
	table.insert(self.Policies, Policies.GoldsellerPolicy(filterData));
	table.insert(self.Policies, Policies.QualityPolicy(filterData));
	table.insert(self.Policies, Policies.SpamPolicy(filterData));

	-- update data, set policy to disabled if not exists in the list
	for _, policy in pairs(self.Policies) do
		if self.Data.Policies[policy.Name] == nil then
			self.Data.Policies[policy.Name] = false;
		end
	end

	Events.AddCallback(Settings, "Saving", function(sender, args)
		args["FilterData"] = self:ExportConfig();
	end);

	Events.AddCallback(Settings, "Loaded", function(sender, args)
		self:ImportConfig(args["FilterData"]);
	end);
end

function FilterEngine:ProcessMessage(message)
	if not self:IsActive() then
		-- not blocked
		return true;
	end

	-- remove tags (risky)
	local plain = function(str)
		return string.gsub(utf8.printable(str), "<[^>]+>", "");
	end

	local plainMessage = plain(message);

	-- the timestamp is not a part of the message (chat option), that's fine
	-- format -> [channel] user: 'text'
	local channel, username, text = string.match(plainMessage, "^%[([^%]]+)%]%s+([^:']+):%s+'(.*)'$");

	if not channel or not username or not text then
		-- if the post is from current player
		-- format -> [To channel] 'text'
		channel, text = string.match(plainMessage, "^%[([^%]]+)%]%s+'(.*)'$");

		if not channel or not text then
			Turbine.Shell.WriteLine("oops, unable to parse the message");
			Turbine.Shell.WriteLine(plainMessage);
			return;
		end

		username = Turbine.Gameplay.LocalPlayer.GetInstance():GetName();
	end

	-- Update stats
	self.Data.NumMessagesTaken = self.Data.NumMessagesTaken + 1;
	
	local bag = {};

	for _, policy in pairs(self.Policies) do
		-- is active?
		if self:IsPolicyEnabled(policy.Name) then

			if policy:Execute(message, channel, username, text, bag) == true then
				-- Blocked
				self:DebugOut("blocked by " .. policy.Name);

				-- Update stats
				self.Data.NumMessagesBlocked = self.Data.NumMessagesBlocked + 1;
				self.Data.BlockedByPolicy[policy.Name] = (self.Data.BlockedByPolicy[policy.Name] or 0) + 1;

				-- Update policies
				self:ApplyResult(message, channel, username, text, true, policy.Name, bag);

				-- Notify
				Events.FireEvent(self, self.MessageBlocked, {});
				Events.FireEvent(self, self.MessageCompleted, {});

				-- blocked
				return false;
			end
		end
	end

	-- Passed

	-- Update policies
	self:ApplyResult(message, channel, username, text, false, nil, bag);

	-- Notify
	Events.FireEvent(self, self.MessagePassed, {});
	Events.FireEvent(self, self.MessageCompleted, {});

	-- passed
	return true;
end

function FilterEngine:ApplyResult(message, channel, username, text, blocked, policyName, bag)
	for _, policy in pairs(self.Policies) do
		if self:IsPolicyEnabled(policy.Name) then
			policy:Apply(message, channel, username, text, blocked, policyName, bag);
		end
	end
end

function FilterEngine:DoHandleChatType(chatType)
	for k, channel in pairs(self.Data.Channels) do
		if channel == chatType then
			return true;
		end
	end

	return false;
end

function FilterEngine:DebugOut(message)
	if self.Data.Debug then
		Turbine.Shell.WriteLine(message);
	end
end

function FilterEngine:IsDebugEnabled()
	return self.Data.Debug;
end

function FilterEngine:SetDebug(value)
	if self.Data.Debug ~= value then
		self.Data.Debug = value;
		Events.FireEvent(self, self.DebugChanged, value);
	end
end

function FilterEngine:DebugChanged()
end

function FilterEngine:IsTraceSpamEnabled()
	return self.Data.TraceSpam;
end

function FilterEngine:SetTraceSpam(value)
	if self.Data.TraceSpam ~= value then
		self.Data.TraceSpam = value;
		Events.FireEvent(self, self.TraceSpamChanged, value);
	end
end

function FilterEngine:TraceSpamChanged()
end

function FilterEngine:IsActive()
	return self.Data.Active;
end

function FilterEngine:SetActive(value)
	if self.Data.Active ~= value then
		self.Data.Active = value;
		Events.FireEvent(self, self.ActiveChanged, value);
	end
end

function FilterEngine:IsPolicyEnabled(policyName)
	return self.Data.Policies[policyName] == true;
end

function FilterEngine:SetPolicyEnabled(policyName, value)
	if self.Data.Policies[policyName] == nil then
		-- not configured
		return;
	end

	if self.Data.Policies[policyName] ~= value then
		self.Data.Policies[policyName] = value
		Events.FireEvent(self, self.PolicyStateChanged, {["Name"] = policyName, ["Value"] = value});
	end
end

function FilterEngine:GetPolicy(policyName)
	for _, policy in pairs(self.Policies) do
		if policy.Name == policyName then
			return policy;
		end
	end
end

function FilterEngine:ActiveChanged(args)
end

function FilterEngine:MessageBlocked(args)
end

function FilterEngine:MessagePassed(args)
end

function FilterEngine:PolicyStateChanged(args)
end

function FilterEngine:ExportConfig()
	local config = {
		["Debug"]     = self.Data.Debug,
		["TraceSpam"] = self.Data.TraceSpam,
		["Active"]    = self.Data.Active,
		["Policies"]  = self.Data.Policies,
	};
	
	return config;
end

function FilterEngine:ImportConfig(config)
	if config == nil then
		return;
	end

	self:SetDebug(config.Debug == true);
	self:SetTraceSpam(config.TraceSpam ~= false);
	self:SetActive(config.Active ~= false);

	if config.Policies then
		for k, v in pairs(config.Policies) do
			self:SetPolicyEnabled(k, v ~= false);
		end
	end
end