
_G.ContentControl = class(LayoutBase);

function ContentControl:Constructor()
	LayoutBase.Constructor(self);

	self.Content = nil;
	self.ContentStyle = ContentStyle();
	self:SetSize(0, 0);
	self:SetWantsUpdates(false);
end

function ContentControl:SetContent(control)
	self:Clear();

	if self.Content then
		if type(self.Content.Destroy) == "function" then
			self.Content:Destroy();
		else
			self.Content:SetParent(nil);
		end

		for k, _ in pairs(self.Content) do
			self.Content[k] = nil;
		end

		self.Content = nil;
	end

    self.Content = control;
	self.Content:SetParent(self);
	self.Content:SetPosition(self.ContentStyle.Padding.Left, self.ContentStyle.Padding.Top);
	self.ContentStyle:SetMinimumSize(self.Content:GetWidth(), self.Content:GetHeight());

	self:RouteEvent(self.Content, "CheckedChanged");
	self:RouteEvent(self.Content, "EnabledChanged");
	self:RouteEvent(self.Content, "MouseClick");
	self:RouteEvent(self.Content, "MouseDown");
	self:RouteEvent(self.Content, "MouseUp");

	return self;
end

function ContentControl:RouteEvent(control, event)
	self._registrations = self._registrations or {};
	
	-- helper
	local function add(object, event, callback)
		Events.AddCallback(object, event, callback);
		table.insert(self._registrations, {["object"] = object, ["event"] = event, ["callback"] = callback});
	end

	-- route self
	add(self, event, function(sender, args)
		if type(self[event]) == "table" then
			-- there is another handler attached, maybe it comes later
			--return;
		end

		local parent = self:GetParent();

		while parent ~= nil do
			local handler = parent[event];

			if handler ~= nil then
				Events.FireEvent(parent, handler, args);
				break;
			end

			parent = parent:GetParent();
		end
	end);

	-- route child content control
	add(control, event, function(sender, args)
		if type(control[event]) == "table" then
			-- there is another handler attached, maybe it comes later
			return;
		end

		-- fire route self event
		Events.FireEvent(self, self[event], args);
	end);
end

function ContentControl:Clear()
	if self._registrations then
		for _, reg in pairs(self._registrations) do
			Events.RemoveCallback(reg.object, reg.event, reg.callback);
		end

		self._registrations = nil;
	end

	LayoutBase.Clear(self);
end

function ContentControl:SetSize(width, height)
	width = math.max(width, self:GetMinWidth());
	height = math.max(height, self:GetMinHeight());

	LayoutBase.SetSize(self, width, height);
end

function ContentControl:SetWidth(width)
	width = math.max(width, self:GetMinWidth());

	LayoutBase.SetWidth(self, width);
end

function ContentControl:SetHeight(height)
	height = math.max(height, self:GetMinHeight());
	
	LayoutBase.SetHeight(self, height);
end

function ContentControl:GetMinWidth()
	return self.ContentStyle.Size.MinimumWidth + self.ContentStyle.Padding.Left + self.ContentStyle.Padding.Right;
end

function ContentControl:GetMinHeight()
	return self.ContentStyle.Size.MinimumHeight + self.ContentStyle.Padding.Top + self.ContentStyle.Padding.Bottom;
end

function ContentControl:GetMaxWidth()
	return math.max(self.ContentStyle.Size.MinimumWidth, self.ContentStyle.Size.MaximumWidth) + self.ContentStyle.Padding.Left + self.ContentStyle.Padding.Right;
end

function ContentControl:GetMaxHeight()
	return math.max(self.ContentStyle.Size.MinimumHeight, self.ContentStyle.Size.MaximumHeight) + self.ContentStyle.Padding.Top + self.ContentStyle.Padding.Bottom;
end

function ContentControl:Layout(args)
	if self.Content == nil then
		-- not set
		return;
	end

	local selfContentWidth, selfContentHeight = self:GetContentSize();

	-- min width
	selfContentWidth = math.max(selfContentWidth, self.ContentStyle.Size.MinimumWidth);

	if not (self.ContentStyle.Size.DynamicWidth > 0) then
		-- max width
		if self.ContentStyle.Size.MaximumWidth > 0 then
			selfContentWidth = math.min(selfContentWidth, self.ContentStyle.Size.MaximumWidth);
		end
	end

	-- min height
	selfContentHeight = math.max(selfContentHeight, self.ContentStyle.Size.MinimumHeight);

	if not (self.ContentStyle.Size.DynamicHeight > 0) then
		-- max height
		if self.ContentStyle.Size.MaximumHeight > 0 then
			selfContentHeight = math.min(selfContentHeight, self.ContentStyle.Size.MaximumHeight);
		end
	end
	
	local contentWidth = selfContentWidth;
	local contentHeight = selfContentHeight;

	if self.ContentStyle.Size.MaximumWidth > 0 then
		contentWidth = math.min(contentWidth, self.ContentStyle.Size.MaximumWidth);
	end
	
	if self.ContentStyle.Size.MaximumHeight > 0 then
		contentHeight = math.min(contentHeight, self.ContentStyle.Size.MaximumHeight);
	end

	-- try to apply the values
	self.Content:SetSize(contentWidth, contentHeight);
	(self.Content.PerformLayout or function()end)(self.Content);
	local realContentWidth, realContentHeight = self.Content:GetSize();

	-- grow to fit the content width
	if not (self.ContentStyle.Size.DynamicWidth > 0) then
		selfContentWidth = math.max(realContentWidth, self.ContentStyle.Size.MinimumWidth);
	else
		selfContentWidth = math.max(realContentWidth, selfContentWidth);
	end

	-- grow to fit the content height
	if not (self.ContentStyle.Size.DynamicHeight > 0) then
		selfContentHeight = math.max(realContentHeight, self.ContentStyle.Size.MinimumHeight);
	else
		selfContentHeight = math.max(realContentHeight, selfContentHeight);
	end

	local contentLeft = self.ContentStyle.Padding.Left;
	local contentTop = self.ContentStyle.Padding.Top;

	-- horizontal alignment
	if self.ContentStyle.HorizontalAlignment == HorizontalAlignment.Left then
		contentLeft = self.ContentStyle.Padding.Left;
	end

	if self.ContentStyle.HorizontalAlignment == HorizontalAlignment.Right then
		contentLeft = (selfContentWidth - realContentWidth) + self.ContentStyle.Padding.Left;
	end

	if self.ContentStyle.HorizontalAlignment == HorizontalAlignment.Center then
		contentLeft = (selfContentWidth - realContentWidth) / 2 +  self.ContentStyle.Padding.Left;
	end

	-- vertical alignment
	if self.ContentStyle.VerticalAlignment == VerticalAlignment.Top then
		contentTop = self.ContentStyle.Padding.Top;
	end

	if self.ContentStyle.VerticalAlignment == VerticalAlignment.Bottom then
		contentTop = (selfContentHeight - realContentHeight) + self.ContentStyle.Padding.Top;
	end

	if self.ContentStyle.VerticalAlignment == VerticalAlignment.Center then
		contentTop = (selfContentHeight - realContentHeight) / 2 +  self.ContentStyle.Padding.Top;
	end

	self.Content:SetPosition(contentLeft, contentTop);
	self:SetContentSize(selfContentWidth, selfContentHeight);
	self:PerformLayout();
end

function ContentControl:GetContentWidth()
	return self:GetWidth() - self.ContentStyle.Padding.Left - self.ContentStyle.Padding.Right;
end

function ContentControl:GetContentHeight()
	return self:GetHeight() - self.ContentStyle.Padding.Top - self.ContentStyle.Padding.Bottom;
end

function ContentControl:GetContentSize()
	return self:GetContentWidth(), self:GetContentHeight();
end

function ContentControl:SetContentWidth(width)
	width = math.max(width, self.ContentStyle.Size.MinimumWidth);
	self:SetWidth(width + self.ContentStyle.Padding.Left + self.ContentStyle.Padding.Right);
end

function ContentControl:SetContentHeight(height)
	height = math.max(height, self.ContentStyle.Size.MinimumHeight);
	self:SetHeight(height + self.ContentStyle.Padding.Top + self.ContentStyle.Padding.Bottom);
end

function ContentControl:SetContentSize(width, height)
	width = math.max(width, self.ContentStyle.Size.MinimumWidth);
	height = math.max(height, self.ContentStyle.Size.MinimumHeight);

	local selfWidth = width + self.ContentStyle.Padding.Left + self.ContentStyle.Padding.Right;
	local selfHeight = height + self.ContentStyle.Padding.Top + self.ContentStyle.Padding.Bottom;
	
	self:SetSize(selfWidth, selfHeight);
end

function ContentControl:SetPadding(value)
	if type(value) == "table" then
		self.ContentStyle.Padding.Left   = math.max(0, value.Left   or self.ContentStyle.Padding.Left);
		self.ContentStyle.Padding.Top    = math.max(0, value.Top    or self.ContentStyle.Padding.Top);
		self.ContentStyle.Padding.Right  = math.max(0, value.Right  or self.ContentStyle.Padding.Right);
		self.ContentStyle.Padding.Bottom = math.max(0, value.Bottom or self.ContentStyle.Padding.Bottom);
		return;
	end

	value = math.max(0, value or 0);
	self.ContentStyle:SetPadding(value, value, value, value);
end

function ContentControl:ApplyContentStyle(contentStyle)
	if contentStyle == nil then
		return;
	end

	-- set up style
	self.ContentStyle:SetPadding(contentStyle.Padding.Left, contentStyle.Padding.Top, contentStyle.Padding.Right, contentStyle.Padding.Bottom);
	self.ContentStyle:SetBackColor(contentStyle.BackColor);
	self.ContentStyle:SetDynamicSize(contentStyle.Size.DynamicWidth, contentStyle.Size.DynamicHeight);
	
	local minWidth = math.max(self.ContentStyle.Size.MinimumWidth, contentStyle.Size.MinimumWidth);
	local minHeight = math.max(self.ContentStyle.Size.MinimumHeight, contentStyle.Size.MinimumHeight);
	self.ContentStyle:SetMinimumSize(minWidth, minHeight);

	-- apply style
	self:SetBackColor(self.ContentStyle.BackColor);

	if self.Content ~= nil then
		self.Content:SetPosition(self.ContentStyle.Padding.Left, self.ContentStyle.Padding.Top);
		self:SetContentSize(self:GetContentSize());
	end
end
