
_G.StackPanel = class(LayoutBase);

function StackPanel:Constructor()
	LayoutBase.Constructor(self);

	self.IsVertical = true;
	self.FitToContent = true;
	self.ChildControls = {};
	self:SetSize(0, 0);
	self:SetWantsUpdates(false);
end

function StackPanel:AddControl(control)
	local content = ContentControl();
	content:SetParent(self);
	content:SetContent(control);
	table.insert(self.ChildControls, content);
	return content;
end

function StackPanel:SetFitToContent(value)
	self.FitToContent = (value == true);
end

function StackPanel:GetMinHeight()
	local parent = self:GetParent();

	if parent and type(parent.ContentStyle) == "table" then
		return parent.ContentStyle.Size.MinimumHeight;
	end

	return 0;
end

function StackPanel:GetMinWidth()
	local parent = self:GetParent();
	
	if parent and type(parent.ContentStyle) == "table" then
		return parent.ContentStyle.Size.MinimumWidth;
	end

	return 0;
end

function StackPanel:Layout()
	-- one of them will match
	self:LayoutVertical();
	self:LayoutHorizontal();
end

-- w, h, w
function StackPanel:LayoutVertical()
	if self.IsVertical ~= true then
		return;
	end

	local selfWidth = math.max(self:GetMinWidth(), self:GetWidth());
	local width = selfWidth;

	-- apply current width
	for _, child in pairs(self.ChildControls) do
		local minWidth = math.min(child:GetMinWidth(), child:GetWidth());
		width = math.max(width, minWidth);
	end

	for _, child in pairs(self.ChildControls) do
		if self.FitToContent then
			child:SetSize(width, child:GetMinHeight());
		else
			child:SetWidth(width);
		end
	end
	
	if self.FitToContent then
		-- calculated later
		width = 0;
	end

	local selfHeight = math.max(self:GetMinHeight(), self:GetHeight());
	local height = 0;
	local sumDynamicHeight = 0;
	
	-- accumulate values
	for _, child in pairs(self.ChildControls) do
		sumDynamicHeight = sumDynamicHeight + math.max(0, child.ContentStyle.Size.DynamicHeight);

		if child.ContentStyle.Size.DynamicHeight > 0 then
			--height = height + child:GetMinHeight();
		else
			child:PerformLayout();
			height = height + child:GetHeight();
		end
	end

	-- Note: the available space can be positive or negative.
	local availableSpace = selfHeight - height;
	local topPosition = 0;

	for _, child in pairs(self.ChildControls) do
		child:SetPosition(0, topPosition);

		if child.ContentStyle.Size.DynamicHeight > 0 then
			-- try set height from dynamic space
			local factor = child.ContentStyle.Size.DynamicHeight / sumDynamicHeight;
			local space = math.floor(factor * availableSpace);

			child:SetHeight(space);
			child:PerformLayout();

			sumDynamicHeight = sumDynamicHeight - child.ContentStyle.Size.DynamicHeight;
			availableSpace = availableSpace - child:GetHeight();
		end

		topPosition = topPosition + child:GetHeight();
		width = math.max(width, child:GetWidth());
	end

	for _, child in pairs(self.ChildControls) do
		child:SetWidth(width);
		child:PerformLayout();
	end

	height = math.max(selfHeight, topPosition);
	
	if self.FitToContent == true then
		height = topPosition;
	end
	
	self:SetSize(width, height);
	--self:PerformLayout();
end

-- h, w, h
function StackPanel:LayoutHorizontal()
	if self.IsVertical == true then
		return;
	end
	
	local selfHeight = math.max(self:GetMinHeight(), self:GetHeight());
	local height = selfHeight;

	-- apply current height
	for _, child in pairs(self.ChildControls) do
		local minHeight = math.min(child:GetMinHeight(), child:GetHeight());
		height = math.max(height, minHeight);
	end
	
	for _, child in pairs(self.ChildControls) do
		if self.FitToContent then
			child:SetSize(child:GetMinWidth(), height);
		else
			child:SetHeight(height);
		end
	end

	if self.FitToContent then
		-- calculated later
		height = 0;
	end

	local selfWidth = math.max(self:GetMinWidth(), self:GetWidth());
	local width = 0;
	local sumDynamicWidth = 0;

	-- accumulate values
	for _, child in pairs(self.ChildControls) do
		sumDynamicWidth = sumDynamicWidth + math.max(0, child.ContentStyle.Size.DynamicWidth);

		if child.ContentStyle.Size.DynamicWidth > 0 then
			--width = width + child:GetMinWidth();
		else
			child:PerformLayout();
			width = width + child:GetWidth();
		end
	end

	-- Note: the available space can be positive or negative.
	local availableSpace = selfWidth - width;
	local leftPosition = 0;

	for _, child in pairs(self.ChildControls) do
		child:SetPosition(leftPosition, 0);

		if child.ContentStyle.Size.DynamicWidth > 0 then
			-- try set width from dynamic space
			local factor = child.ContentStyle.Size.DynamicWidth / sumDynamicWidth;
			local space = math.floor(factor * availableSpace);
			
			child:SetWidth(space);
			child:PerformLayout();

			sumDynamicWidth = sumDynamicWidth - child.ContentStyle.Size.DynamicWidth;
			availableSpace = availableSpace - child:GetWidth();
		end

		leftPosition = leftPosition + child:GetWidth();
		height = math.max(height, child:GetHeight());
	end

	for _, child in pairs(self.ChildControls) do
		child:SetHeight(height);
		child:PerformLayout();
	end

	width = math.max(selfWidth, leftPosition);
	
	if self.FitToContent == true then
		width = leftPosition;
	end

	self:SetSize(width, height);
	--self:PerformLayout();
end

