
MonitorWindow = class(BaseWindow);

function MonitorWindow:Constructor(filterEngine)
	BaseWindow.Constructor(self);
	self.Engine = filterEngine;
	
	local content = TemplateLoader.Setup(self, self:GetTemplate());
	self:SetContent(content);
	self:RegisterCallbacks();
	self:SetText("Monitor");
	self:FitToContent();
	self:SetMaximumSize(1000, 790);
	self:UpdateStats();
end

function MonitorWindow:RegisterCallbacks()
	self._registrations = self._registrations or {};

	-- helper
	local function add(object, event, callback)
		Events.AddCallback(object, event, callback);
		table.insert(self._registrations, {["object"] = object, ["event"] = event, ["callback"] = callback});
	end

	add(self.Engine, "MessageCompleted", function(sender, args)
		self:UpdateStats(args);
	end);
end

function MonitorWindow:UnregisterCallbacks()
	if self._registrations then
		for _, reg in pairs(self._registrations) do
			Events.RemoveCallback(reg.object, reg.event, reg.callback);
		end

		self._registrations = nil;
	end
end

function MonitorWindow:Closing(args)
	self:UnregisterCallbacks();
	self.Engine = nil;
	BaseWindow.Closing(self, args);
end

function MonitorWindow:UpdateStats(args)
	self.MemoryTextBlock:SetText(string.format("%.2f", collectgarbage("count") / 1024) .. " MB");

	if self["ChatQualityTextBlock"] == nil then
		local bindings = {["Title"] = "Chat Quality", ["TextBlockName"] = "ChatQualityTextBlock"};
		local item = TemplateLoader.Setup(self, self:GetStatsItemTemplate(bindings));
		local control = self.StatsPanel:AddControl(item);
		control.ContentStyle.VerticalAlignment = VerticalAlignment.Center;
	end

	self.ChatQualityTextBlock:SetText(string.format("%.2f", self.Engine.Data.ChatQuality));
	
	if self["ChatMessageLengthTextBlock"] == nil then
		local bindings = {["Title"] = "Chat Message Length", ["TextBlockName"] = "ChatMessageLengthTextBlock"};
		local item = TemplateLoader.Setup(self, self:GetStatsItemTemplate(bindings));
		local control = self.StatsPanel:AddControl(item);
		control.ContentStyle.VerticalAlignment = VerticalAlignment.Center;
	end

	self.ChatMessageLengthTextBlock:SetText(string.format("%.2f", self.Engine.Data.ChatMessageLength));

	if self["MessagesTakenTextBlock"] == nil then
		local bindings = {["Title"] = "Messages Taken", ["TextBlockName"] = "MessagesTakenTextBlock"};
		local item = TemplateLoader.Setup(self, self:GetStatsItemTemplate(bindings));
		local control = self.StatsPanel:AddControl(item);
		control.ContentStyle.VerticalAlignment = VerticalAlignment.Center;
	end

	self.MessagesTakenTextBlock:SetText(self.Engine.Data.NumMessagesTaken);

	if self["MessagesBlockedTextBlock"] == nil then
		local bindings = {["Title"] = "Messages Blocked", ["TextBlockName"] = "MessagesBlockedTextBlock"};
		local item = TemplateLoader.Setup(self, self:GetStatsItemTemplate(bindings));
		local control = self.StatsPanel:AddControl(item);
		control.ContentStyle.VerticalAlignment = VerticalAlignment.Center;
	end

	self.MessagesBlockedTextBlock:SetText(self.Engine.Data.NumMessagesBlocked);

	for k, v in pairs(self.Engine.Data.BlockedByPolicy) do
		local name = k .. "TextBlock";

		if self[name] == nil then
			local bindings = {["Title"] = "Blocked by " .. k, ["TextBlockName"] = name};
			local item = TemplateLoader.Setup(self, self:GetStatsItemTemplate(bindings));
			local control = self.StatsPanel:AddControl(item);
			control.ContentStyle.VerticalAlignment = VerticalAlignment.Center;
		end

		self[name]:SetText(v);
	end

	self:Invalidate();
end

function MonitorWindow:GetTemplate(bindings)
	return {
		["Type"] = Border,
		["Width"] = 300,
		["Height"] = 200,
		["BorderWidth"] = 1,
		["Content"] = 
		{
			{
				["Type"] = Image,
				["ImageMode"] = ImageMode.KeepImage,
				["VerticalAlignment"] = VerticalAlignment.Center,
				["HorizontalAlignment"] = HorizontalAlignment.Center,
				["Background"] = Images.Monitor,
				["BlendMode"] = Turbine.UI.BlendMode.Multiply,
				["ImageBackColor"] = Turbine.UI.Color(0.98, 0.2, 0.2, 0.2),
				["Content"] = 
				{
					{
						["Type"] = ScrollViewer,
						["Content.DynamicHeight"] = 1,
						["Content.DynamicWidth"] = 1,
						["Content"] =
						{
							{
								["Type"] = StackPanel,
								["Name"] = "StatsPanel",
								["Content"] =
								{
									{
										["Type"] = StackPanel,
										["IsVertical"] = false,
										["FitToContent"] = false,
										["Content.VerticalAlignment"] = VerticalAlignment.Center,
										["Content"] =
										{
											{
												["Type"] = TextBlock,
												["Text"] = "Memory",
												["Content.DynamicWidth"] = 1,
												["Content.Padding"] = 4,
											},
											{
												["Type"] = TextBlock,
												["Name"] = "MemoryTextBlock",
												["Content.DynamicWidth"] = 1,
												["Content.Padding"] = 4,
											},
										},
									},
								},
							},
						},
					},
				},
			},
		},
	};
end

function MonitorWindow:GetStatsItemTemplate(bindings)
	return {
		["Type"] = StackPanel,
		["IsVertical"] = false,
		["FitToContent"] = false,
		["Content"] =
		{
			{
				["Type"] = TextBlock,
				["Text"] = bindings["Title"],
				["Content.DynamicWidth"] = 1,
				["Content.Padding"] = 4,
			},
			{
				["Type"] = TextBlock,
				["Name"] = bindings["TextBlockName"],
				["Content.DynamicWidth"] = 1,
				["Content.Padding"] = 4,
			},
		},
	};
end
