
OptionsPanel = class(LayoutBase);

function OptionsPanel:Constructor(filterEngine)
	LayoutBase.Constructor(self);
	self.Engine = filterEngine;
	self:Initialize();
end

function OptionsPanel:Initialize()
	-- set global styles
	ButtonStyle.BorderWidth = 1;
	ButtonStyle.Font = Turbine.UI.Lotro.Font.TrajanPro14; -- supported, see Fonts
	ButtonStyle.ForeColor = Turbine.UI.Color(0.7, 0.7, 0.2);
	ButtonStyle.Image = Images.Button;
	ButtonStyle.ImageMode = ImageMode.RepeatImage;
	ButtonStyle.FontStyle = Turbine.UI.FontStyle.Outline;
	ButtonStyle.MouseDownColor = Turbine.UI.Color(0.98, 0.3, 0.3, 0.3);
	ButtonStyle.MouseEnterColor = Turbine.UI.Color(0.98, 0.7, 0.7, 1);
	ButtonStyle.MouseLeaveColor = Turbine.UI.Color(0.98, 0.4, 0.4, 0.4);

	self.Refs = {};

	local openWindow = function(window)
		self.Refs[window] = window;

		local function windowClosed(sender, args)
			Events.RemoveCallback(sender, "Closed", windowClosed);
			self.Refs[sender] = nil;
		end

		Events.AddCallback(window, "Closed", windowClosed);

		window:CenterWindow();
		window:SetVisible(true);
	end

	local bindings = {
		["IsDebug"] = self.Engine:IsDebugEnabled(),
		["DebugCheckedChanged"] = function(sender, args)
			self.Engine:SetDebug(sender:IsChecked());
		end,
		
		["IsTraceSpam"] = self.Engine:IsTraceSpamEnabled(),
		["TraceSpamCheckedChanged"] = function(sender, args)
			self.Engine:SetTraceSpam(sender:IsChecked());
		end,

		["IsActive"] = self.Engine:IsActive(),
		["ActiveCheckedChanged"] = function(sender, args)
			self.Engine:SetActive(sender:IsChecked());
		end,

		["MonitorButtonClick"] = function() 
			openWindow(MonitorWindow(self.Engine));
		end,
		
		["SpamPolicyButtonClick"] = function() 
			openWindow(SpamPolicyWindow(self.Engine));
		end,

		["BlacklistPolicyButtonClick"] = function() 
			openWindow(BlacklistPolicyWindow(self.Engine));
		end,

		["GoldsellerPolicyButtonClick"] = function() 
			openWindow(GoldsellerPolicyWindow(self.Engine));
		end,
		
		["HelpButtonClick"] = function() 
			openWindow(HelpWindow());
		end,
	};

	-- content
	local content = TemplateLoader.Setup(self, self:GetTemplate(bindings));
	content:SetParent(self);

	-- add policies
	for k, v in pairs(self.Engine.Data.Policies) do
		local item;
		local bindings;
		
		bindings = {
			["CheckBoxName"] = k,
			["Title"] = k,
			["Checked"] = v,
			["CheckedChanged"] = function(sender, args)
				self.Engine:SetPolicyEnabled(k, sender:IsChecked());
			end,
		};

		item = TemplateLoader.Setup(self, self:GetPolicyItemTemplate(bindings));

		local control = self.PoliciesPanel:AddControl(item);
		control.ContentStyle:SetPadding(4);
	end

	-- add callbacks
	self:RegisterCallbacks();
end

function OptionsPanel:RegisterCallbacks()
	self._registrations = self._registrations or {};
	
	-- helper
	local function add(object, event, callback)
		Events.AddCallback(object, event, callback);
		table.insert(self._registrations, {["object"] = object, ["event"] = event, ["callback"] = callback});
	end

	add(self.Engine, "DebugChanged", function(sender, args)
		self.DebugCheckBox:SetChecked(args);
	end);

	add(self.Engine, "TraceSpamChanged", function(sender, args)
		self.TraceSpamCheckBox:SetChecked(args);
	end);

	add(self.Engine, "ActiveChanged", function(sender, args)
		self.ActiveCheckBox:SetChecked(args);
	end);

	add(self.Engine, "PolicyStateChanged", function(sender, args)
		if self[args.Name] ~= nil then
			self[args.Name]:SetChecked(args.Value);
		end
	end);
end

function OptionsPanel:UnregisterCallbacks()
	if self._registrations then
		for _, reg in pairs(self._registrations) do
			Events.RemoveCallback(reg.object, reg.event, reg.callback);
		end

		self._registrations = nil;
	end
end

function OptionsPanel:SizeChanged(args)
	LayoutBase.SizeChanged(self, args);
	self:PerformLayout();
end

function OptionsPanel:Layout()
	self.Content:SetSize(self:GetSize());
	self.Content:PerformLayout();
	self:SetSize(self.Content:GetSize());
	--self:PerformLayout();
end

function OptionsPanel:GetTemplate(bindings)
	return {
		["Type"] = Border,
		["Name"] = "Content",
		["BorderWidth"] = 10,
		["BorderColor"] = Turbine.UI.Color(0.9, 0.0, 0.0, 0.0),
		["Content"] = 
		{
			{
				["Type"] = StackPanel,
				["Content"] =
				{
					{
						["Type"] = Image,
						["Height"] = 150,
						["ImageMode"] = ImageMode.KeepImage,
						["Background"] = Images.Headline,
						["BlendMode"] = Turbine.UI.BlendMode.Overlay,
						["HorizontalAlignment"] = HorizontalAlignment.Center,
						["VerticalAlignment"] = VerticalAlignment.Center,
					},
					{
						["Type"] = TextBlock,
						["Height"] = 50,
						["Text"] = "Einstellungen",
						["TextAlignment"] = Turbine.UI.ContentAlignment.MiddleCenter,
						["Content.VerticalAlignment"] = VerticalAlignment.Center,
					},
					{
						["Type"] = Border,
						["BorderWidth"] = 1,
						["Content"] =
						{
							{
								["Type"] = StackPanel,
								["Content.Padding"] = 4,
								["Content"] =
								{
									{
										["Type"] = CheckBox,
										["Name"] = "ActiveCheckBox",
										["Text"] = "Spam-Filter aktivieren",
										["Checked"] = bindings["IsActive"],
										["CheckedChanged"] = bindings["ActiveCheckedChanged"],
										["Content.Padding"] = 4,
									},
									{
										["Type"] = CheckBox,
										["Name"] = "TraceSpamCheckBox",
										["Text"] = "Kurze <rgb=#ffffff>[Spam]</rgb> <rgb=#ff0000>[Welt] Text ...</rgb> Nachricht ausgeben",
										["Checked"] = bindings["IsTraceSpam"],
										["CheckedChanged"] = bindings["TraceSpamCheckedChanged"],
										["Content.Padding"] = 4,
									},
									{
										["Type"] = CheckBox,
										["Name"] = "DebugCheckBox",
										["Text"] = "Debug",
										["Checked"] = bindings["IsDebug"],
										["CheckedChanged"] = bindings["DebugCheckedChanged"],
										["Content.Padding"] = 4,
									},
								},
							},
						},
					},
					{
						["Type"] = Border,
						["BorderWidth"] = 1,
						["Content.PaddingTop"] = 10,
						["Content"] =
						{
							{
								["Type"] = StackPanel,
								["Name"] = "PoliciesPanel",
								["Content.Padding"] = 4,
								["Content"] =
								{
								},
							},
						},
					},
					{
						["Type"] = FlowPanel,
						["Content.PaddingTop"] = 10,
						["Content"] =
						{
							{
								["Type"] = Button,
								["Text"] = "Monitor",
								["Content.MaxWidth"] = 150,
								["Content.PaddingRight"] = 4,
								["Content.PaddingBottom"] = 4,
								["Click"] = bindings["MonitorButtonClick"],
							},
							{
								["Type"] = Button,
								["Text"] = "Spam Policy",
								["Content.MaxWidth"] = 150,
								["Content.PaddingRight"] = 4,
								["Content.PaddingBottom"] = 4,
								["Click"] = bindings["SpamPolicyButtonClick"],
							},
							{
								["Type"] = Button,
								["Text"] = "Blacklist Policy",
								["Content.MaxWidth"] = 150,
								["Content.PaddingRight"] = 4,
								["Content.PaddingBottom"] = 4,
								["Click"] = bindings["BlacklistPolicyButtonClick"],
							},
							{
								["Type"] = Button,
								["Text"] = "Goldseller Policy",
								["Content.MaxWidth"] = 150,
								["Content.PaddingRight"] = 4,
								["Content.PaddingBottom"] = 4,
								["Click"] = bindings["GoldsellerPolicyButtonClick"],
							},
							{
								["Type"] = Button,
								["Text"] = "Hilfe",
								["Content.MaxWidth"] = 150,
								["Content.PaddingRight"] = 4,
								["Content.PaddingBottom"] = 4,
								["Click"] = bindings["HelpButtonClick"],
							},
						},
					},
				},
			},
		},
	};
end

function OptionsPanel:GetPolicyItemTemplate(bindings)
	return {
		["Type"] = CheckBox,
		["Name"] = bindings["CheckBoxName"],
		["Text"] = bindings["Title"],
		["Checked"] = bindings["Checked"],
		["CheckedChanged"] = bindings["CheckedChanged"],
	};
end
